@@ -27,6 +27,7 @@
 #define NDPI_CURRENT_PROTO NDPI_PROTOCOL_GUILDWARS
 
 #include "ndpi_api.h"
+#include "ndpi_private.h"
 
 
 static void ndpi_int_guildwars_add_connection(struct ndpi_detection_module_struct *ndpi_struct, struct ndpi_flow_struct *flow)
@@ -34,45 +35,45 @@ static void ndpi_int_guildwars_add_connection(struct ndpi_detection_module_struc
   ndpi_set_detected_protocol(ndpi_struct, flow, NDPI_PROTOCOL_GUILDWARS, NDPI_PROTOCOL_UNKNOWN, NDPI_CONFIDENCE_DPI);
 }
 
-void ndpi_search_guildwars_tcp(struct ndpi_detection_module_struct *ndpi_struct, struct ndpi_flow_struct *flow)
+static void ndpi_search_guildwars_tcp(struct ndpi_detection_module_struct *ndpi_struct, struct ndpi_flow_struct *flow)
 {
-	struct ndpi_packet_struct *packet = &ndpi_struct->packet;
+  struct ndpi_packet_struct *packet = &ndpi_struct->packet;
 
-	NDPI_LOG_DBG(ndpi_struct, "search guildwars\n");
+  NDPI_LOG_DBG(ndpi_struct, "search guildwars\n");
 
-	if (packet->payload_packet_len == 64 && get_u_int16_t(packet->payload, 1) == ntohs(0x050c)
-		&& memcmp(&packet->payload[50], "@2&P", 4) == 0) {
-		NDPI_LOG_INFO(ndpi_struct, "found GuildWars version 29.350\n");
-		ndpi_int_guildwars_add_connection(ndpi_struct, flow);
-		return;
-	}
-	if (packet->payload_packet_len == 16 && get_u_int16_t(packet->payload, 1) == ntohs(0x040c)
-		&& get_u_int16_t(packet->payload, 4) == ntohs(0xa672)
-		&& packet->payload[8] == 0x01 && packet->payload[12] == 0x04) {
-		NDPI_LOG_INFO(ndpi_struct, "found GuildWars version 29.350\n");
-		ndpi_int_guildwars_add_connection(ndpi_struct, flow);
-		return;
-	}
-	if (packet->payload_packet_len == 21 && get_u_int16_t(packet->payload, 0) == ntohs(0x0100)
-		&& get_u_int32_t(packet->payload, 5) == ntohl(0xf1001000)
-		&& packet->payload[9] == 0x01) {
-		NDPI_LOG_INFO(ndpi_struct, "found GuildWars version 216.107.245.50\n");
-		ndpi_int_guildwars_add_connection(ndpi_struct, flow);
-		return;
-	}
+  if (packet->payload_packet_len == 64 && get_u_int16_t(packet->payload, 1) == ntohs(0x050c)
+      && memcmp(&packet->payload[50], "@2&P", 4) == 0) {
+    NDPI_LOG_INFO(ndpi_struct, "found GuildWars version 29.350\n");
+    ndpi_int_guildwars_add_connection(ndpi_struct, flow);
+    return;
+  }
+  if (packet->payload_packet_len == 16 && get_u_int16_t(packet->payload, 1) == ntohs(0x040c)
+      && get_u_int16_t(packet->payload, 4) == ntohs(0xa672)
+      && packet->payload[8] == 0x01 && packet->payload[12] == 0x04) {
+    NDPI_LOG_INFO(ndpi_struct, "found GuildWars version 29.350\n");
+    ndpi_int_guildwars_add_connection(ndpi_struct, flow);
+    return;
+  }
+  if (packet->payload_packet_len == 21 && get_u_int16_t(packet->payload, 0) == ntohs(0x0100)
+      && get_u_int32_t(packet->payload, 5) == ntohl(0xf1001000)
+      && packet->payload[9] == 0x01) {
+    NDPI_LOG_INFO(ndpi_struct, "found GuildWars version 216.107.245.50\n");
+    ndpi_int_guildwars_add_connection(ndpi_struct, flow);
+    return;
+  }
 
-	NDPI_EXCLUDE_PROTO(ndpi_struct, flow);
+  NDPI_EXCLUDE_PROTO(ndpi_struct, flow);
 }
 
 
-void init_guildwars_dissector(struct ndpi_detection_module_struct *ndpi_struct, u_int32_t *id, NDPI_PROTOCOL_BITMASK *detection_bitmask)
+void init_guildwars_dissector(struct ndpi_detection_module_struct *ndpi_struct, u_int32_t *id)
 {
-    ndpi_set_bitmask_protocol_detection("Guildwars", ndpi_struct, detection_bitmask, *id,
+  ndpi_set_bitmask_protocol_detection("Guildwars", ndpi_struct, *id,
 				      NDPI_PROTOCOL_GUILDWARS,
 				      ndpi_search_guildwars_tcp,
 				      NDPI_SELECTION_BITMASK_PROTOCOL_V4_V6_TCP_WITH_PAYLOAD_WITHOUT_RETRANSMISSION,
 				      SAVE_DETECTION_BITMASK_AS_UNKNOWN,
 				      ADD_TO_DETECTION_BITMASK);
 
-    *id += 1;
+  *id += 1;
 }
