@@ -28,6 +28,7 @@
 #define NDPI_CURRENT_PROTO NDPI_PROTOCOL_TELNET
 
 #include "ndpi_api.h"
+#include "ndpi_private.h"
 
 /* #define TELNET_DEBUG 1 */
 
@@ -39,7 +40,8 @@ static int search_telnet_again(struct ndpi_detection_module_struct *ndpi_struct,
   int i;
 
 #ifdef TELNET_DEBUG
-  printf("==> %s() [%s][direction: %u]\n", __FUNCTION__, packet->payload, packet->packet_direction);
+  printf("==> %s() [%.*s][direction: %u]\n", __FUNCTION__, packet->payload_packet_len,
+	 packet->payload, packet->packet_direction);
 #endif
   
   if((packet->payload == NULL)
@@ -63,7 +65,7 @@ static int search_telnet_again(struct ndpi_detection_module_struct *ndpi_struct,
 	return(1);
 	
       flow->protos.telnet.password_detected = 1;
-      ndpi_set_risk(ndpi_struct, flow, NDPI_CLEAR_TEXT_CREDENTIALS);
+      ndpi_set_risk(flow, NDPI_CLEAR_TEXT_CREDENTIALS, "Found password");
       flow->protos.telnet.password[flow->protos.telnet.character_id] = '\0';
       return(0);
     }
@@ -89,17 +91,35 @@ static int search_telnet_again(struct ndpi_detection_module_struct *ndpi_struct,
   }
 
   if(packet->payload[0] == '\r') {
+    char buf[64];
+    
     flow->protos.telnet.username_detected = 1;
-    ndpi_set_risk(ndpi_struct, flow, NDPI_CLEAR_TEXT_CREDENTIALS);
     flow->protos.telnet.username[flow->protos.telnet.character_id] = '\0';
     flow->protos.telnet.character_id = 0;
+
+    snprintf(buf, sizeof(buf), "Found Telnet username (%s)",
+	     flow->protos.telnet.username);
+    ndpi_set_risk(flow, NDPI_CLEAR_TEXT_CREDENTIALS, buf);
+
     return(1);
   }
 
   for(i=0; i<packet->payload_packet_len; i++) {
     if(packet->packet_direction == 0) /* client -> server */ {
       if(flow->protos.telnet.character_id < (sizeof(flow->protos.telnet.username)-1))
-	flow->protos.telnet.username[flow->protos.telnet.character_id++] = packet->payload[i];
+      {
+        if (i>=packet->payload_packet_len-2 &&
+            (packet->payload[i] == '\r' || packet->payload[i] == '\n'))
+        {
+          continue;
+        }
+        else if (ndpi_isprint(packet->payload[i]) == 0)
+        {
+          flow->protos.telnet.username[flow->protos.telnet.character_id++] = '?';
+        } else {
+          flow->protos.telnet.username[flow->protos.telnet.character_id++] = packet->payload[i];
+        }
+      }
     }
   }
 
@@ -111,10 +131,6 @@ static int search_telnet_again(struct ndpi_detection_module_struct *ndpi_struct,
 
 static void ndpi_int_telnet_add_connection(struct ndpi_detection_module_struct
 					   *ndpi_struct, struct ndpi_flow_struct *flow) {
-  flow->guessed_host_protocol_id = flow->guessed_protocol_id = NDPI_PROTOCOL_TELNET;
-
-  /* This is necessary to inform the core to call this dissector again */
-  flow->check_extra_packets = 1;
   flow->max_extra_packets_to_check = 64;
   flow->extra_packets_func = search_telnet_again;
 
@@ -130,8 +146,7 @@ __mingw_forceinline static
 #else
 __forceinline static
 #endif
-u_int8_t search_iac(struct ndpi_detection_module_struct *ndpi_struct,
-		    struct ndpi_flow_struct *flow) {
+u_int8_t search_iac(struct ndpi_detection_module_struct *ndpi_struct) {
   struct ndpi_packet_struct *packet = &ndpi_struct->packet;
 
   u_int16_t a;
@@ -169,11 +184,11 @@ u_int8_t search_iac(struct ndpi_detection_module_struct *ndpi_struct,
 /* ************************************************************************ */
 
 /* this detection also works asymmetrically */
-void ndpi_search_telnet_tcp(struct ndpi_detection_module_struct *ndpi_struct,
-			    struct ndpi_flow_struct *flow) {
+static void ndpi_search_telnet_tcp(struct ndpi_detection_module_struct *ndpi_struct,
+				   struct ndpi_flow_struct *flow) {
   NDPI_LOG_DBG(ndpi_struct, "search telnet\n");
 
-  if(search_iac(ndpi_struct, flow) == 1) {
+  if(search_iac(ndpi_struct) == 1) {
     if(flow->l4.tcp.telnet_stage == 2) {
       NDPI_LOG_INFO(ndpi_struct, "found telnet\n");
       ndpi_int_telnet_add_connection(ndpi_struct, flow);
@@ -201,9 +216,9 @@ void ndpi_search_telnet_tcp(struct ndpi_detection_module_struct *ndpi_struct,
 }
 
 
-void init_telnet_dissector(struct ndpi_detection_module_struct *ndpi_struct, u_int32_t *id, NDPI_PROTOCOL_BITMASK *detection_bitmask)
+void init_telnet_dissector(struct ndpi_detection_module_struct *ndpi_struct, u_int32_t *id)
 {
-  ndpi_set_bitmask_protocol_detection("Telnet", ndpi_struct, detection_bitmask, *id,
+  ndpi_set_bitmask_protocol_detection("Telnet", ndpi_struct, *id,
 				      NDPI_PROTOCOL_TELNET,
 				      ndpi_search_telnet_tcp,
 				      NDPI_SELECTION_BITMASK_PROTOCOL_V4_V6_TCP_WITH_PAYLOAD_WITHOUT_RETRANSMISSION,
